/*
 * Copyright (c) 1994/95 by Jaroslav Kysela (Perex soft)
 */

#ifndef __GUS_H
#define __GUS_H

#if defined( LINUX ) || defined( __LINUX__ )
#include <linux/ioctl.h>
#endif

#define GUS_CARDS		8	/* number of supported GUS cards */

/*
 *  Section for direct support of GF1 synthesizer - /dev/gus
 */

#define GUS_SYNTH_VERSION	0x0501
	/* current version of protocol - 5.01; main mask is 0xff00 */
	/* changes in low 8 bites -> protocols are backwards compatible */
	/* high 8. bites -> protocols are totaly uncompatible */

/* commands over ioctl function - control */
 
#define GUS_IOCTL_CARDS		_IOR ( 'g', 0x00, int )
			/* return number of installed cards */
#define GUS_IOCTL_CARD_SELECT	_IOW ( 'g', 0x00, int )
			/* set card */
#define GUS_IOCTL_VERSION	_IOR ( 'g', 0x01, int )
			/* return version of protocol (current GUS_SYNTH_VERSION) */
#define GUS_IOCTL_RESET0	_IOR ( 'g', 0x02, struct GUS_STRU_RESET )
			/* reset synth engine only - not GF1 */
#define GUS_IOCTL_RESET		_IOWR( 'g', 0x02, struct GUS_STRU_RESET )
			/* first arg is number of active voices 14-32 */
#define GUS_IOCTL_INFO		_IOR ( 'g', 0x03, struct GUS_STRU_INFO )
			/* arg is ptr to GUS_STRU_INFO */

/* commands over ioctl function - setup */

#define GUS_IOCTL_VOL_TYPE	_IOW ( 'g', 0x20, int )
			/* volume type - see GUS_VOL_TYPE_XXXX constants */
#define GUS_IOCTL_FREQ_TYPE	_IOW ( 'g', 0x21, int )
			/* frequency type - see GUS_FREQ_TYPE_XXXX constants */
#define GUS_IOCTL_SET_AUTO_RAMP	_IOW ( 'g', 0x22, int )
			/* set auto ramp speed, 0 = off */
#define GUS_IOCTL_GET_AUTO_RAMP _IOR ( 'g', 0x22, int )
			/* get auto ramp speed */
#define GUS_IOCTL_SET_SPAN	_IOW ( 'g', 0x23, int )
			/* set smooth pan speed in 80 us */
#define GUS_IOCTL_GET_SPAN	_IOR ( 'g', 0x23, int )
			/* get smooth pan speed in 80 us */
#define GUS_IOCTL_NOISE_CHANNEL	_IOW ( 'g', 0x24, struct GUS_STRU_NOISE )

/* commands over ioctl function - timer */

#define GUS_IOCTL_START_TIMER	_IO  ( 'g', 0x40 )
			/* arg is count in one us */
#define GUS_IOCTL_STOP_TIMER	_IO  ( 'g', 0x41 )
			/* stop timer */
#define GUS_IOCTL_INS_GTIMER	_IO  ( 'g', 0x48 )
			/* insert this card to global timer */
#define GUS_IOCTL_DEL_GTIMER	_IO  ( 'g', 0x49 )
			/* remove this card from global timer */

/* commands over ioctl function - samples */

#define GUS_IOCTL_DOWNLOAD	_IOW ( 'g', 0x60, struct GUS_STRU_DOWNLOAD )
			/* arg is ptr to GUS_STRU_DOWNLOAD */
#define GUS_IOCTL_DOWNLOAD_RESET _IO  ( 'g', 0x61 )
			/* reset GF1 memory manager */
#define GUS_IOCTL_DOWNLOAD_TEST _IOW ( 'g', 0x62, struct GUS_STRU_DOWNLOAD )
			/* arg is ptr to GUS_STRU_DOWNLOAD */
#define GUS_IOCTL_DOWNLOAD_TEST1 _IOW ( 'g', 0x63, struct GUS_STRU_DOWNLOAD )
			/* arg is ptr to GUS_STRU_DOWNLOAD */

/* commands over ioctl function - command queue */

#define GUS_IOCTL_QSIZE		_IOR ( 'g', 0x80, int )
			/* get queue size in items (ticks) */
#define GUS_IOCTL_QFREE		_IOR ( 'g', 0x81, int )
			/* get free queue items (ticks) */
#define GUS_IOCTL_THRESHOLD	_IOW ( 'g', 0x82, int )
			/* set min wait free queue items to wakeup */
#define GUS_IOCTL_FLUSH		_IO  ( 'g', 0x83 )
			/* flush command queue */
#define GUS_IOCTL_ABORT		_IO  ( 'g', 0x84 )
			/* abort command queue and stop gf1 */
#define GUS_IOCTL_QABORT	_IO  ( 'g', 0x85 )
			/* abort only command queue to command STOP */
#define GUS_IOCTL_STOP		_IO  ( 'g', 0x86 )
			/* stop playing */
#define GUS_IOCTL_CONTINUE	_IO  ( 'g', 0x87 )
			/* continue playing */
#define GUS_IOCTL_SET_TICK	_IOW ( 'g', 0x88, unsigned int )
			/* set current absolute song tick */
#define GUS_IOCTL_GET_TICK	_IOR ( 'g', 0x88, unsigned int )
			/* get current absolute song tick */
#define GUS_IOCTL_ECHO_ITEMS	_IOW ( 'g', 0x89, int )
			/* set buffer for echo items - if 0 - erase buffer */

/*
 *  error codes
 */
 
#define EBADVOICE	322000
#define EBADSAMPLE	322001
#define EBADRAMP	322002
#define EECHOFULL	322003

/*
 *  commands in write queue
 *
 *  format of queue:
 *  
 *    unsigned char	size		size of command
 *    unsigned char	command		see GUS_CMD_XXXX constants
 *    unsigned char	[size-2]	command argument bytes
 */

#define GUS_CMD_VOICE_SELECT	0x01	/* voice select - (unsigned char) 0-31 */
#define GUS_CMD_SAMPLE_SELECT	0x02	/* sample select - (unsigned short) 0-(GUS_MAX_SAMPLES-1) */

#define GUS_CMD_VOICE_CONTROL	0x10	/* voice control register (unsigned char) */
#define GUS_CMD_FREQUENCY	0x11	/* voice frequency (unsigned short) */
#define GUS_CMD_LOOP_START	0x12	/* voice loop start (long) - relative to begin of sample */
#define GUS_CMD_LOOP_END	0x13	/* voice loop end (long) - relative to begin of sample */
#define GUS_CMD_RAMP_RATE	0x14	/* voice ramp rate (unsigned char) */
#define GUS_CMD_RAMP_START	0x15	/* voice ramp start (unsigned char) */
#define GUS_CMD_RAMP_END	0x16	/* voice ramp end (unsigned char) */
#define GUS_CMD_CURRENT_VOL	0x17	/* voice current volume (unsigned short) */
#define GUS_CMD_CURRENT_LOC	0x18	/* voice current location (long) - relative to begin of sample */
#define GUS_CMD_CURRENT_PAN	0x19	/* voice current pan position (unsigned char) */
#define GUS_CMD_RAMP_CONTROL	0x1a	/* voice ramp control (unsigned char) */
#define GUS_CMD_FREQ_AND_VOL	0x1b	/* select voice & freq setup */
#define GUS_CMD_LOOP_ALL	0x1c	/* set all loop parameters: voice control, loop start, loop end */
#define GUS_CMD_RAMP_ALL	0x1d	/* set all ramp parameters: ramp control, ramp rate, ramp start, ramp end */
#define GUS_CMD_VOICE_STOP	0x1e	/* stop voice */

/*
 * commands 0x20-0x2f are equivalents to 0x10-0x1f, but
 * first byte of argument is number of voice (channel)
 */

#define GUS_CMD_VS_VOICE_CONTROL 0x20
#define GUS_CMD_VS_FREQUENCY	0x21
#define GUS_CMD_VS_LOOP_START	0x22
#define GUS_CMD_VS_LOOP_END	0x23
#define GUS_CMD_VS_RAMP_RATE	0x24
#define GUS_CMD_VS_RAMP_START	0x25
#define GUS_CMD_VS_RAMP_END	0x26
#define GUS_CMD_VS_CURRENT_VOL	0x27
#define GUS_CMD_VS_CURRENT_LOC	0x28
#define GUS_CMD_VS_CURRENT_PAN	0x29
#define GUS_CMD_VS_RAMP_CONTROL	0x2a
#define GUS_CMD_VS_FREQ_AND_VOL	0x2b
#define GUS_CMD_VS_LOOP_ALL	0x2c
#define GUS_CMD_VS_RAMP_ALL	0x2d
#define GUS_CMD_VS_VOICE_STOP	0x2e

#define GUS_CMD_SAMPLE_START	0x40	/* voice (short), start sample (short) 0-128, */
 					/* frequency (unsigned short), volume (unsigned short) */
#define GUS_CMD_SAMPLE_START1	0x41

#define GUS_CMD_WAIT		0x50	/* wait x us (int) 1-(int max) */
#define GUS_CMD_STOP		0x51	/* see GUS_IOCTL_ABORT ioctl */
#define GUS_CMD_ECHO		0x52	/* this will be send to file input */

/* bits for voice_cntrl_reg variable in GUS_STRU_SAMPLE */

#define GUS_WAVE_LOOP_BACK	0x0040	/* loop back */
#define GUS_WAVE_BIDIRECTIONAL	0x0010	/* bidirectional loop enable */
#define GUS_WAVE_LOOP		0x0008	/* loop enable */
#define GUS_WAVE_16BIT		0x0004	/* 16 bit sample width */
#define GUS_WAVE_INVERT		0x0001	/* invert high bit */

#define GUS_WAVE_ENVELOPES	0x0100	/* envelopes on */
#define GUS_WAVE_SUSTAIN_ON	0x0200
#define GUS_WAVE_VIBRATO	0x0400
#define GUS_WAVE_TREMOLO	0x0800
#define GUS_WAVE_SCALE		0x1000

/* defines for GUS_IOCTL_VOL_TYPE */

#define GUS_VOL_TYPE_GF1	0x00	/* volume type is GF1 */
#define GUS_VOL_TYPE_LINEAR	0x01	/* 0-128 - linear volume */
#define GUS_VOL_TYPE_LINEAR2	0x02	/* 0-16384 - extended linear volume */

/* defines for GUS_IOCTL_FREQ_TYPE */

#define GUS_FREQ_TYPE_GF1	0x00	/* frequency type - GF1 */
#define GUS_FREQ_TYPE_HZ2	0x01	/* frequency type - HZ * 2 */
#define GUS_FREQ_TYPE_HZ	0x02	/* frequency type - HZ */

/* structures */

struct GUS_STRU_RESET {
  short voices;			  /* active voices */
  short samples;		  /* max samples to be stored in GUS's DRAM */
};

#define GUS_STRU_INFO_VERSION_MASK	0x0fff
#define GUS_STRU_INFO_VERSION_DB16	0x1000	/* flag */

struct GUS_STRU_INFO {
  unsigned short port;
  unsigned short irq;
  unsigned short dma1;		  /* DMA1 - GF1 download & codec record */
  unsigned short dma2;		  /* DMA2 - GF1 record & codec playback */
  unsigned short version;	  /* version of gus 0x24, 0x35, 0x37 or 0xa0 (MAX) or 0x100 (PnP) */
  unsigned int memory_size;	  /* in bytes */
  unsigned int memory_free;	  /* in bytes */
  int max_bank_free;		  /* maximum free memory in bank */
  short noise_channel;		  /* if != 0 - noise channel enabled */
};

struct GUS_STRU_DOWNLOAD {
  unsigned char *sample_ptr;	  /* pointer to sample */
  int loop_start;		  /* start of loop */
  int loop_end;		  	  /* end of loop */
  int length;			  /* size of sample */
  unsigned short mode;		  /* see GUS_WAVE_XXXX constants */

  unsigned char env_rate[ 6 ];
  unsigned char env_offset[ 6 ];
  
  unsigned char tremolo_sweep;
  unsigned char tremolo_rate;
  unsigned char tremolo_depth;
  
  unsigned char vibrato_sweep;
  unsigned char vibrato_rate;
  unsigned char vibrato_depth;
  
  int scale_frequency;
  unsigned int scale_factor;
};

#define GUS_NOISE_NEW_PATTERN	0x0001

struct GUS_STRU_NOISE {
  unsigned short flags;		  /* see to GUS_NOISE_XXXX constants */
  unsigned volume;
  unsigned short frequency;	  /* frequency in GF1 format !!!! */
  				  /* 0x200 = 44kHz - 14 voices */
  /* 
   *  bellow structure members is overwritten by kernel if
   *  GUS_NOISE_NEW_PATTERN isn't set
   */
  unsigned char voice_cntrl_reg;  /* only GUS_WAVE_16BIT is accepted */
  unsigned size;		  /* real size of noise pattern */  				 
  unsigned char pattern[ 14 ];
};

/*
 *  Section for direct support of GF1 synthesizer - /dev/gusctl
 */

#define GUS_SYNTH_CTL_VERSION	GUS_SYNTH_VERSION
	/* current version of protocol; main mask is 0xff00 */
	/* changes in low 8 bites -> protocols are backwards compatible */
	/* high 8. bites -> protocols are totaly uncompatible */
	
/* commands over ioctl function - control */

#define GUS_IOCTL_CTL_CARDS		_IOR ( 'g', 0xa0, int )
			/* get number of cards */
#define GUS_IOCTL_CTL_CARD_SELECT	_IOW ( 'g', 0xa0, int )
			/* select card */
#define GUS_IOCTL_CTL_VERSION		_IOR ( 'g', 0xa1, int )
			/* get version */
#define GUS_IOCTL_CTL_INFO		_IOR ( 'g', 0xa1, struct GUS_STRU_INFO )
			/* get various informations */

/* commands over ioctl function - samples */

#define GUS_IOCTL_CTL_DOWNLOAD_RESET	_IOW ( 'g', 0xb0, int )
			/* arg is number of samples */
#define GUS_IOCTL_CTL_DOWNLOAD_TEST	_IOW ( 'g', 0xb1, struct GUS_STRU_DOWNLOAD )
			/* only test download */
#define GUS_IOCTL_CTL_DOWNLOAD_TEST1	_IOW ( 'g', 0xb2, struct GUS_STRU_DOWNLOAD )
			/* don't remember information about this sample */

/*
 *  Compatibility section for VoxWare 3.00.
 *  VoxWare 3.00 - Copyright (c) by Hannu Savolainen.
 *  Code for gus module driver isn't copied from VoxWare driver - 
 *  It's copyrighted by me (Jaroslav Kysela).
 */

#ifndef SOUND_VERSION
#include <sys/soundcard.h>
#include <sys/ultrasound.h>
#if SOUND_VERSION < 300 && SOUND_VERSION > 301
#error Bad version of VoxWare header files... You must use VoxWare version 3.00-3.01.
#endif
#endif

#if 0
#include <sys/voxware_soundcard.h>
#endif

/*
 *  some mixer extensions
 */

#define SOUND_MIXER_SELECT		0x0f
#define SOUND_MIXER_CARDS		MIXER_READ( SOUND_MIXER_SELECT )
#define SOUND_MIXER_CARD_SELECT		MIXER_WRITE( SOUND_MIXER_SELECT )
#define SOUND_MIXER_GUS_VERSION		0x11
#define SOUND_MIXER_READ_GUS_VERSION	MIXER_READ( SOUND_MIXER_GUS_VERSION )
#define SOUND_MIXER_MUTE_FLAG		0x40000000	/* maybe wrong */
#define SOUND_MIXER_DEVS_LMUTE		0x12
#define SOUND_MIXER_READ_DEVS_LMUTE	MIXER_READ( SOUND_MIXER_DEVS_LMUTE )
#define SOUND_MIXER_WRITE_DEVS_LMUTE	MIXER_WRITE( SOUND_MIXER_DEVS_LMUTE )
#define SOUND_MIXER_DEVS_RMUTE		0x13
#define SOUND_MIXER_READ_DEVS_RMUTE	MIXER_READ( SOUND_MIXER_DEVS_RMUTE )
#define SOUND_MIXER_WRITE_DEVS_RMUTE	MIXER_WRITE( SOUND_MIXER_DEVS_RMUTE )

/*
 *  some PCM extensions
 */
 
#define SOUND_PCM_CARDS			_IOR ( 'P', 240, int )
#define SOUND_PCM_CARD_SELECT		_IOW ( 'P', 240, int )

/*
 *
 */
 
#endif /* __GUS_H */
